﻿using System.IO;
using Abp;
using Abp.Castle.Logging.Log4Net;
using Abp.PlugIns;
using Abp.Reflection.Extensions;
using Castle.Facilities.Logging;
using Castle.Windsor.MsDependencyInjection;
using Microsoft.Extensions.DependencyInjection;
using MyCompanyName.AbpZeroTemplate.Identity;
using Topshelf;

namespace MyCompanyName.AbpZeroTemplate.WinService
{
    class Program
    {
        static void Main(string[] args)
        {
            var currentDirectory = typeof(Program).GetAssembly().GetDirectoryPathOrNull();
            // 设置工作目录. 保证工作在windows service时log文件存放目录正确
            // 如果不设置, 工作在windows service时log文件将会存放在system32目录中
            Directory.SetCurrentDirectory(currentDirectory);

            var services = new ServiceCollection();
            IdentityRegistrar.Register(services);

            var abpBootstrapper = AbpBootstrapper.Create<AbpZeroWinServiceModule>(options =>
            {
                options.IocManager.IocContainer.AddFacility<LoggingFacility>(
                                    f => f.UseAbpLog4Net().WithConfig(Path.Combine(currentDirectory, $"log4net.config"))
                                );
                options.PlugInSources.AddFolder(Path.Combine(currentDirectory, "Plugins"), SearchOption.AllDirectories);
            });

            services.AddSingleton(abpBootstrapper);
            abpBootstrapper.IocManager.IocContainer.AddServices(services);

            HostFactory.Run(x =>
            {
                x.Service<AbpZeroWinService>(s =>
                {
                    s.ConstructUsing(name => new AbpZeroWinService());
                    s.WhenStarted((tc, hostControl) => tc.Start(hostControl));
                    s.WhenStopped((tc, hostControl) => tc.Stop(hostControl));
                });

                x.RunAsLocalSystem();
                x.StartAutomatically();

                x.SetDescription("ABP服务测试");
                x.SetDisplayName("ABPTestService");
                x.SetServiceName("ABPTestService");
            });
        }
    }
}
